/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*! \file GUIInterface.h
    \brief Defines interface for GUI

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/



#ifndef GUIINTERFACE_H
#define GUIINTERFACE_H
#include <QMainWindow>
#include <QMultiHash>

#include "translation.h"
#include "CommonDictInterface.h"

class Settings;

//! Interface for different GUIs
/*!
  Default base class for all GUIs is QMainWindow
  */
class GUIInterface : public QMainWindow {
    Q_OBJECT

public:
    GUIInterface(QWidget *parent = 0) :QMainWindow(parent) {}
    virtual ~GUIInterface() {}

    //! Returns all loaded dictionaries with information if they are
    //! active/inactive
    /*!
        \return Hash of pointers to a dictionary and boolean flag indicating if
        a dictionary is active
     */
    virtual QHash<CommonDictInterface*, bool> getDictionaries() = 0;


    //! Returns all loaded plugins
    /*!
        \return List of pointers to plugins
     */
    virtual QList<CommonDictInterface*> getPlugins() = 0;

    //! Indicates if GUI is in exact search mode.
    /*! When GUI is in exact search mode it searches for a word, and
        if it finds exactly matching translation it directly displays it, without
        displaying matching words list. This mode should be
        used for browsing search history and searching for words from application
        arguments.
        \returns flag indicating if GUI is in exact search mode
    */
    bool exactSearch();

    //! Sets GUI exact search mode.
    /*! \sa exactSearch() */
    void setExactSearch(bool exactSearch);

    virtual Settings* settings() = 0;

    virtual void setSettings(Settings*) = 0;


public Q_SLOTS:
    //! Searches in exact mode for a given word
    /*!
      GUI will be automatically set into exact search mode, and after search or
      break it will be unset from exact search mode.
      \param word word which will be searched for in dictionaries
      */
    virtual void searchExact(QString word) = 0;

    //! Adds key words from given translations to history
    /*!
      By default this slot is connected to searchTranslations signal, and
      passed translations list contains only translations with the same key, so
      only one word is added to history.
      \param list of translations with key words
      \sa searchTranslations();
      */
    virtual void addToHistory(QList<Translation*>) = 0;

    //! Shows history dialog
    virtual void showHistory(QPoint) = 0;

    //! Shows translation of next word in history
    /*!
      It will work only if there is next word available in history.
      Translation of a word is searched for with searchExact() function
      \sa searchExact()
      */
    virtual void historyNext() = 0;

    //! Shows translation of previous word in history
    /*!
      It will work only if there is previous word available in history.
      Translation of a word is searched for with searchExact() function
      \sa searchExact()
      */
    virtual void historyPrev() = 0;

    //! Gets words list from backbone and prepares received list to display
    /*!
      Checks if received list is empty, in that case it displays suitable
      information. If GUI is in exact search mode it will search for exact
      word in received list, and if the word is found it will emit signal to
      display its translation. Otherwise it will display list of matching
      words and show suitable information.
      \sa exactSearch()
      \sa showTranslation()
     */
    virtual void wordListReady() = 0;

    //! Gets translation strings from backbone and emits signal to display them
    virtual void translationsReady() = 0;

Q_SIGNALS:
    //! Should be emitted when user wants to close application to stop
    //! all ongoing searches
    void quit();

    //! Emitted when user wants to search for a list of words matching a given word
    /*! \param word word which will be matched, it can contain wildcards
    */
    void searchWordList(QString word);

    //! Emitted when user wants to see translation of words.
    /*! \param list of translations for a given word which will be received
        in wordListReady() slot
        \sa wordListReady()
    */
    void searchTranslations(QList<Translation*>);

    //! Emitted when starting search, will disable GUI components
    //! and show progress bars
    void setBusy();

    //! Emitted when searching ends, will enable GUI components
    void setIdle();

    //! Emitted when user wants to break search
    void stopSearching();

    //! Emitted after receiving words list in wordListReady() slot, will display
    //! list of matched words
    /*! \param hash of a word and list of translations of this word found
         in dictionaries
     */
    void showWordList(QHash<QString, QList<Translation*> >);

    //! Emitted after receiving translation strings in translationsReady() slot,
    //! will display translation of a given word
    /*! \param list of translations from different dictionaries
         in dictionaries
     */
    void showTranslation(QStringList);

    //! Emitted when user wants to add a new dictionary
    /*! \param new dictionary returned by specific plugin dialog
      */
    void addNewDictionary(CommonDictInterface*);

    //! Emitted when user wants to remove a dictionary
    /*! \param dictionary which will be removed
      */
    void removeDictionary(CommonDictInterface*);

    //! Emitted when user changes dictionaries active/inactive states
    /*! \param list of only active dictionaries
      */
    void selectedDictionaries(QList<CommonDictInterface* >);

    void addToBookmarks(QList<Translation*>);
};

#endif // GUIINTERFACE_H
