/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/

/*! \file DictManagerWidget.cpp
    \brief Implements dictionaries management widget

    \author Mateusz Półrola <mateusz.polrola@comarch.pl>
*/

#include "DictManagerWidget.h"
#include "DictTypeSelectDialog.h"
#include <QDebug>
#include "../../include/DictDialog.h"
#include "MenuWidget.h"

DictManagerWidget::DictManagerWidget(GUIInterface *parent) :
    QDialog(parent) {
    setWindowTitle(tr("Dictionaries"));
    this->guiInterface = parent;

    #ifndef Q_WS_MAEMO_5
    model = 0;
    #endif
    initalizeUI();

    setModal(true);
}

void DictManagerWidget::initalizeUI() {
    verticalLayout = new QVBoxLayout;
    setLayout(verticalLayout);

    #ifndef Q_WS_MAEMO_5
    qmlView = new QDeclarativeView(this);

    ctxt = qmlView->rootContext();

    refreshDictsList();
    ctxt->setContextProperty("dictModel", &(*model));
    qmlView->setSource(QUrl::fromLocalFile("/usr/share/mdictionary/qml/DictManagerWidget.qml"));

    QGraphicsObject *rootObject = qmlView->rootObject();

    qmlView->setResizeMode(QDeclarativeView::SizeRootObjectToView);
    verticalLayout->addWidget(qmlView);

    #endif

    #ifdef Q_WS_MAEMO_5
    dictList = new QListWidget;
    verticalLayout->addWidget(dictList);

    dictList->setSelectionMode(QAbstractItemView::SingleSelection);
    dictList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    addNewDictButton = new QPushButton(tr("Add"));
    removeDictButton = new QPushButton(tr("Remove"));
    settingsButton = new QPushButton(tr("Settings"));

    removeDictButton->setEnabled(false);
    settingsButton->setEnabled(false);

    buttonGroup = new QHBoxLayout;

    buttonGroup->addWidget(addNewDictButton);
    buttonGroup->addWidget(removeDictButton);
    buttonGroup->addWidget(settingsButton);

    verticalLayout->addLayout(buttonGroup, Qt::AlignBottom);


    connect(addNewDictButton, SIGNAL(clicked()),
            this, SLOT(saveChanges()));
    connect(addNewDictButton, SIGNAL(clicked()),
            this, SLOT(addNewDictButtonClicked()));

    connect(removeDictButton, SIGNAL(clicked()),
            this, SLOT(saveChanges()));
    connect(removeDictButton, SIGNAL(clicked()),
            this, SLOT(removeButtonClicked()));

    connect(settingsButton, SIGNAL(clicked()),
            this, SLOT(saveChanges()));
    connect(settingsButton, SIGNAL(clicked()),
            this, SLOT(settingsButtonClicked()));

    connect(dictList, SIGNAL(itemClicked(QListWidgetItem*)),
            this, SLOT(itemSelected(QListWidgetItem*)));

    connect(dictList, SIGNAL(itemChanged(QListWidgetItem*)),
            this, SLOT(changed()));
    #endif

    #ifndef Q_WS_MAEMO_5
    connect(rootObject, SIGNAL(addButtonClicked()),
            this, SLOT(saveChanges()));
    connect(rootObject, SIGNAL(addButtonClicked()),
            this, SLOT(addNewDictButtonClicked()));

    connect(rootObject, SIGNAL(removeButtonClicked()),
            this, SLOT(saveChanges()));
    connect(rootObject, SIGNAL(removeButtonClicked()),
            this, SLOT(removeButtonClicked()));

    connect(rootObject, SIGNAL(settingsButtonClicked()),
            this, SLOT(saveChanges()));
    connect(rootObject, SIGNAL(settingsButtonClicked()),
            this, SLOT(settingsButtonClicked()));


    connect(model, SIGNAL(itemChanged()),
            this, SLOT(changed()));
    connect(rootObject, SIGNAL(itemActivated(int)),
            this, SLOT(saveChanges()));
    connect(rootObject, SIGNAL(itemActivated(int)),
            this, SLOT(settingsButtonClicked()));

    #endif

    #ifdef Q_WS_MAEMO_5
    refreshDictsList();
    #endif

    #ifndef Q_WS_MAEMO_5
        setMinimumSize(500,300);
        //closeButton = new QPushButton(tr("Save"));
        //buttonGroup->addWidget(closeButton);

//        setMinimumWidth(sizeHint().width()*1.2);
//        setMaximumWidth(sizeHint().width()*2);
//        setMinimumHeight(sizeHint().height());
//        setMaximumHeight(sizeHint().height()*2);
        connect(rootObject, SIGNAL(saveButtonClicked()), this, SLOT(save()));
    #endif
}


void DictManagerWidget::refreshDictsList() {

    #ifndef Q_WS_MAEMO_5
    QHash<CommonDictInterface*, bool> dicts = guiInterface->getDictionaries();

    if (model == 0){
        model = new DictManagerModel(dicts, this);
    } else {
        model->clear();
        model->setDictionaries(dicts);
    }

    #endif

    #ifdef Q_WS_MAEMO_5
    dictList->clear();
    dictsHash.clear();
    removeDictButton->setEnabled(false);
    settingsButton->setEnabled(false);

    QHash<CommonDictInterface*, bool> dicts = guiInterface->getDictionaries();

    QHashIterator<CommonDictInterface*, bool> i(dicts);

    while(i.hasNext()) {
        i.next();

        QListWidgetItem* item = new QListWidgetItem();
        QString name;
        if (i.key()->type() == "stardict") {
            name = i.key()->name() + " (" + i.key()->type() + ")";
        }
        else {
            name = i.key()->langFrom() + " - " + i.key()->langTo() +
                   " (" + i.key()->type() + " " +
                   i.key()->name() + ")";
        }
        item->setText(name);
        item->setFlags(item->flags() | Qt::ItemIsUserCheckable);
        if(i.value()) {
            item->setCheckState(Qt::Checked);
        }
        else {
            item->setCheckState(Qt::Unchecked);
        }
        item->setIcon(*i.key()->icon());

        dictList->addItem(item);
        dictsHash.insert(item, i.key());
    }

    dictList->sortItems();
    #endif
}

void DictManagerWidget::showEvent(QShowEvent *e) {
    _changed = false;
    #ifndef Q_WS_MAEMO_5
      _save = false;
    #endif
    refreshDictsList();
    QWidget::showEvent(e);
}

void DictManagerWidget::saveChanges() {

    #ifndef Q_WS_MAEMO_5
        if(_save) {
    #else
        if(_changed &&
                QMessageBox::question(this, tr("Save"),
                                      tr("Do you want to save changes?"),
                QMessageBox::Save, QMessageBox::Cancel) == QMessageBox::Save) {
            QList<CommonDictInterface*> checkedDicts;

            for(int i=0; i<dictList->count(); i++) {
                QListWidgetItem* item = dictList->item(i);
                if(item->checkState() == Qt::Checked) {
                    checkedDicts.push_back(dictsHash[item]);
                }
            }
    #endif
        QList<CommonDictInterface*> checkedDicts;

        for(int i=0; i<model->dictList().count(); i++) {
//            QListWidgetItem* item = dictList->item(i);
            CommonDictInterface* item = model->dictList()[i];
            if(model->dictionaries()[item] == true) {
                checkedDicts.push_back(item);
            }
        }
        _changed = false;
        emit selectedDictionaries(checkedDicts);
    }
}

void DictManagerWidget::hideEvent(QHideEvent *e) {
    saveChanges();
    QWidget::hideEvent(e);
}


void DictManagerWidget::addNewDictButtonClicked() {
    #ifndef Q_WS_MAEMO_5
    qDebug()<<"1 "<<_changed;
    if(!_changed || QMessageBox::question(this,
            tr("Save"), tr("Do you want to save changes?"),
            QMessageBox::Save, QMessageBox::Cancel) == QMessageBox::Save) {
        qDebug()<<"2";
        _save = true;
        saveChanges();
        _save = false;
    }
    #endif

   CommonDictInterface* selectedPlugin =
           DictTypeSelectDialog::addNewDict(guiInterface->getPlugins(),this);
   if(selectedPlugin) {
       Settings* settings =
               selectedPlugin->dictDialog()->addNewDictionary(this->parentWidget());

       if(settings) {
           CommonDictInterface* newDict = selectedPlugin->getNew(settings);
           if(newDict) {
               delete settings;
               Q_EMIT addDictionary(newDict);
           }
       }
   }
   refreshDictsList();
}

void DictManagerWidget::itemSelected(QListWidgetItem *) {
    removeDictButton->setEnabled(true);
    settingsButton->setEnabled(true);
    dictList->setFocus();
}

void DictManagerWidget::removeButtonClicked() {
    if(QMessageBox::question(this, tr("Remove dictionary"),
            tr("Do you want to remove selected dictionary?"),
            QMessageBox::Yes, QMessageBox::No) == QMessageBox::Yes) {
#ifndef Q_WS_MAEMO_5
        emit removeDictionary(model->currentDict());
        refreshDictsList();
#else
        QList<QListWidgetItem*> selected = dictList->selectedItems();
        if(selected.count() > 0) {
            emit removeDictionary(dictsHash[selected[0]]);
            refreshDictsList();
        }
#endif
   }
}

void DictManagerWidget::settingsButtonClicked() {
    #ifndef Q_WS_MAEMO_5
    if(!_changed || QMessageBox::question(this,
            tr("Save"), tr("Do you want to save changes?"),
            QMessageBox::Save, QMessageBox::Cancel) == QMessageBox::Save) {
        _save = true;
        saveChanges();
        _save = false;
    }
    model->currentDict()->dictDialog()->changeSettings(this->parentWidget());
    refreshDictsList();
    #else
   QList<QListWidgetItem*> selected = dictList->selectedItems();
   if(selected.count() > 0) {
       dictsHash[selected[0]]->dictDialog()->changeSettings(this->parentWidget());
   }
   refreshDictsList();
#endif
}


void DictManagerWidget::changed() {
    _changed=true;
}


#ifndef Q_WS_MAEMO_5
    void DictManagerWidget::save() {
        _save = true;
        hide();
    }
#endif


void DictManagerWidget::keyPressEvent(QKeyEvent *e) {
    if (!e->modifiers() || (e->modifiers() & Qt::KeypadModifier && e->key() == Qt::Key_Enter)) {
        switch (e->key()) {
            case Qt::Key_Escape:
               reject();
               break;
            default:
               e->ignore();
               return;
        }
    } else {
        e->ignore();
    }
}
