/*******************************************************************************

    This file is part of mDictionary.

    mDictionary is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    mDictionary is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with mDictionary.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2010 Comarch S.A.

*******************************************************************************/
/*!
     \file StarDictPlugin.cpp
     \brief Implementation of stardict plugin's main class.

     \author Jakub Jaszczynski
*/

#include "StarDictPlugin.h"

/**/
uint qHash(const TranslationStarDict &key) {
   return qHash(key.key());
}


StarDictPlugin::StarDictPlugin(QObject *parent) : CommonDictInterface(parent),
                    _langFrom(""), _langTo(""),_name(""), _infoNote("") {
    _settings = new Settings();
    _dictDialog = new StarDictDialog(this, this);

    connect(_dictDialog, SIGNAL(notify(Notify::NotifyType,QString)),
            this, SIGNAL(notify(Notify::NotifyType,QString)));

    _settings->setValue("type","stardict");
    _iconPath = "/usr/share/mdictionary/stardict.png";
    _icon = QIcon(_iconPath);
    stopped = false;

    _settings->setValue("strip_accents","true");

    initAccents();

}


void StarDictPlugin::retranslate() {
    QString locale = QLocale::system().name();
    QTranslator *translator = new QTranslator(this);

    if(!translator->load(":/stardict/translations/" + locale)) {
        translator->load(":/stardict/translations/en_US");
    }
    QCoreApplication::installTranslator(translator);
}


StarDictPlugin::~StarDictPlugin() {
    delete _settings;
    delete _dictDialog;
}


QString StarDictPlugin::langFrom() const {
    return _langFrom;
}


QString StarDictPlugin::langTo() const {
    return  _langTo;
}


QString StarDictPlugin::name() const {
    return  _name;
}


QString StarDictPlugin::type() const {
    return QString("stardict");
}


QString StarDictPlugin::infoNote() const {
    return _name;
}


QList<Translation*> StarDictPlugin::searchWordList(QString word, int limit) {
    QList<TranslationStarDict> translations;
    bool is32b = false;
    bool err = 0;
    int wordcount = settings()->value("wordcount").toInt(&err);
    int counter = 0;
    int counterLimit = 0;
    QString idxPath = settings()->value("idxFileName");
    StarDictReader * reader = StarDictReaderFactory::createReader(idxPath);
    QString fkey;
    qint64 offset = 0,
    len = 0;

    if( word.indexOf("*")==-1 && word.indexOf("?")==-1 &&
        word.indexOf("_")==-1 && word.indexOf("%")==-1)
        word+="*";
    QRegExp keyword(word, Qt::CaseInsensitive, QRegExp::Wildcard);

    if((settings()->value("idxoffsetbits") == "32") ||
                         (settings()->value("idxoffsetbits") == ""))
        is32b = true;

    if(!err)
        return QList<Translation*>();

    stopped=false;
    while(counter < wordcount && (counterLimit<limit || limit==0) && !stopped) {
        counter++;
        fkey = reader->readKeyword();
        if(is32b)
            offset = reader->readInt32BigEndian();
        else
            offset = reader->readInt64BigEndian();
        len = reader->readInt32BigEndian();

        if(keyword.exactMatch(fkey) ) {
            TranslationStarDict tran(fkey, infoNote(), this);
  //        qDebug() << "off/len" << offset << len;
            int id = translations.indexOf(tran);
            if(id == -1) {
                tran.add(offset, len);
                translations.push_front(tran);
            } else
                translations[id].add(offset, len);
            counterLimit++;
        }
    }
    QList<Translation*> ret;
    QListIterator<TranslationStarDict> it(translations);
    while(it.hasNext())
        ret.push_back(new TranslationStarDict(it.next()));
    return ret;
}


QByteArray StarDictPlugin::read(QByteArray::iterator it,
        QByteArray::iterator end, int bytes) {
    QByteArray ret;

    if(bytes == 0 && it != end){
        while(*it != '\0' && it != end)
            ret.append(*it++);
    }
    else
        for(int i = 0; i < bytes && it != end; i++)
            ret.append(*it++);
    return ret;
}


QString StarDictPlugin::interpret(QByteArray::iterator it,
        QByteArray::iterator end, QChar mode,QString key, bool) {
    QString result;
//    qDebug()<<"****** mode:     "<<mode;
    if(mode == 'm'
            || mode == 'l'
            || mode == 'g'
            || mode == 't'
            || mode == 'y'
            || mode == 'k'
            || mode == 'w'
            || mode == 'h'
            || mode == 'r'){
        result += "<key>" + key + "</key>";
        result += "<t>" + QString::fromUtf8(read(it++, end)) + "</t>";
    }

    if(mode ==  'g'){
        /* delete "small" tag  from g translation*/
        int indexOfSmall=result.indexOf("<small>");
        while(indexOfSmall!=-1){
            int indexOfEndSmall= result.indexOf("</small>");
            if(indexOfEndSmall!=-1)
                result.remove(indexOfSmall,indexOfEndSmall-indexOfSmall+8);
            indexOfSmall=result.indexOf("<small>");
        }
    }
    if(mode == 'x'){
        result += QString::fromUtf8(read(it++, end));
        result.replace("</k>","</key><t>");
        result.replace("<k>","</t><key>");
        int pos=result.indexOf("</t>");
        if(pos!=-1)
            result.remove(pos,4);
        if(result.contains("<t>"))
            result+="</t>";
    }

// W & P Dont know whether mDictionary would ever handle binary stardict format
// to be honest dont see any kind of adventages (can't find any binary dict)
 /*
    }
    else if(mode == 'W') {
        result += "<key>" + key + "</key>";
        if(!last) {
            QByteArray tmp ;
            tmp.append(*(it++));
            tmp.append(*(it++));
            tmp.append(*(it++));
            tmp.append(*(it));
            result += read(it++, end, (qint32)qFromBigEndian(*(qint32*)tmp.data()));
        } else
            result += read(it++, end);
    } else if(mode == 'P') {
        result += "<key>" + key + "</key>";
        if(!last) {
            QByteArray tmp ;
            tmp.append(*(it++));
            tmp.append(*(it++));
            tmp.append(*(it++));
            tmp.append(*(it));
            result += read(it++, end, (qint32)qFromBigEndian(*(qint32*)tmp.data()));
        } else
            result += read(it++, end);
    } */
    return result;
}


QString StarDictPlugin::format(QByteArray raw, QString mode,QString key) {
    QString result;
    if(mode == "") {
        for(QByteArray::iterator it = raw.begin(); it != raw.end(); it++) {
            char tmp = *(++it);
            result += interpret(--it, raw.end(), tmp, key);
        }
    } else {
        QByteArray::iterator it = raw.begin();
        foreach(QChar tmp, mode) {
            result += interpret(it, raw.end(), tmp, key);
        }
    }
    return result;
}


QString StarDictPlugin::search(QString key, qint64 offset, qint32 len) {
    QString dictPath = settings()->value("dictFileName");
    StarDictReader *reader = StarDictReaderFactory::createReader(dictPath);
    QByteArray raw = reader->readString(offset, len);
    QString result= format(raw, settings()->value("sametypesequence"),key);

    delete reader;
    return result;
}


void StarDictPlugin::stop() {
    stopped=true;
}


DictDialog* StarDictPlugin::dictDialog() {
     return _dictDialog;
}


CommonDictInterface* StarDictPlugin::getNew(const Settings *settings) const {
    StarDictPlugin *plugin = new StarDictPlugin();

    connect(plugin, SIGNAL(notify(Notify::NotifyType,QString)),
            this, SIGNAL(notify(Notify::NotifyType,QString)));

    ((StarDictDialog*)plugin->dictDialog())->
            setLastDialogParent(_dictDialog->lastDialogParent());

    if(settings && plugin->setSettings(settings)) {
        disconnect( plugin, SIGNAL(notify(Notify::NotifyType,QString)),
                    this, SIGNAL(notify(Notify::NotifyType,QString)));
        plugin->getDictionaryInfo();
        return plugin;
    }
    else {
        disconnect( plugin, SIGNAL(notify(Notify::NotifyType,QString)),
                    this, SIGNAL(notify(Notify::NotifyType,QString)));
        delete plugin;
        return 0;
    }
}


bool StarDictPlugin::isAvailable() const {
    return true;
}


Settings* StarDictPlugin::settings() {
    return _settings;
}


bool StarDictPlugin::setSettings(const Settings *sett) {
    if(sett) {
        foreach(QString key, sett->keys())
            _settings->setValue(key, sett->value(key));

    } else
        return false;
    Q_EMIT settingsChanged();
    return true;
}


bool StarDictPlugin::getDictionaryInfo() {
    QFile file(settings()->value("ifoFileName"));
    if(!QFile::exists(_settings->value("ifoFileName"))
                || !file.open(QFile::ReadOnly | QFile::Text)) {
        Q_EMIT notify(Notify::Warning,
               QString(tr("StarDict dictionary cannot be read from file")));
        qDebug()<<"Error: could not open the file";
        return false;
    }
    QTextStream in(&file);
    while (!in.atEnd()) {
        QString line = in.readLine();
        QStringList list = line.split("=");
        if(list.size() == 2) {
            settings()->setValue(list.at(0),list.at(1));
        }
    }
    _name = settings()->value("bookname");
    return true;
}


QIcon* StarDictPlugin::icon() {
    return &_icon;
}

QString StarDictPlugin::iconPath(){
    return _iconPath;
}

Q_EXPORT_PLUGIN2(stardict, StarDictPlugin)
